// File from page 704 in "Thinking in C++" by Bruce Eckel
//////////////////////////////////////////////////
// From the compressed package ECKELT02.ZIP 4/11/95
// (Original ECKELT01.ZIP dated 2/21/95)
// Copyright (c) Bruce Eckel, 1995 
// Source code file from the book "Thinking in C++", 
// Prentice Hall, 1995, ISBN: 0-13-917709-4
// All rights reserved EXCEPT as allowed by the following 
// statements: You may freely use this file for your own 
// work, including modifications and distribution in 
// executable form only. You may copy and distribute this 
// file, as long as it is only distributed in the complete 
// (compressed) package with the other files from this 
// book and you do not remove this copyright and notice. 
// You may not distribute modified versions of the source 
// code in this package. This package may be freely placed 
// on bulletin boards, internet nodes, shareware disks and 
// product vendor disks. You may not use this file in 
// printed media without the express permission of the 
// author. Bruce Eckel makes no 
// representation about the suitability of this software 
// for any purpose. It is provided "as is" without express 
// or implied warranty of any kind. The entire risk as to 
// the quality and performance of the software is with 
// you. Should the software prove defective, you assume 
// the cost of all necessary servicing, repair, or 
// correction. 
// If you think you've found an error, please 
// email all modified files with loudly commented changes 
// to: eckel@aol.com (please use the same 
// address for non-code errors found in the book).
//////////////////////////////////////////////////

//: WRAPPED.CPP -- Safe, atomic pointers
#include <fstream.h>
#include <stdlib.h>
ofstream out("wrapped.out");

// Simplified. Yours may have other arguments.
template<class T, int sz = 1> class pwrap {
  T* ptr;
public:
  class rangeError {}; // Exception class
  pwrap() {
    ptr = new T[sz];
    out << "pwrap constructor" << endl;
  }
  ~pwrap() {
    delete []ptr;
    out << "pwrap destructor" << endl;
  }
  T& operator[](int i) throw(rangeError) {
    if(i >= 0 && i < sz) return ptr[i];
    throw rangeError();
  }
};

class bonk {
public:
  bonk() { out << "bonk()" << endl; }
  ~bonk() { out << "~bonk()" << endl; }
  void g() {}
};

class og {
public:
  void* operator new[](size_t sz) {
    out << "allocating an og" << endl;
    throw int(47);
    return 0;
  }
  void operator delete[](void* p) {
    out << "deallocating an og" << endl;
    ::delete p;
  }
};

class useResources {
  pwrap<bonk, 3> Bonk;
  pwrap<og> Og;
public:
  useResources() : Bonk(), Og() {
    out << "useResources()" << endl;
  }
  ~useResources() {
    out << "~useResources()" << endl;
  }
  void f() { Bonk[1].g(); }
};

main() {
  try {
    useResources ur;
  } catch(int) {
    out << "inside handler" << endl;
  } catch(...) {
    out << "inside catch(...)" << endl;
  }
}
